import 'dart:convert';

// until goes into weebni_models
// meanwhile

// item : qt 2 , panier de légumes (lots : 1.1.1 & 2.1.1), [ 1.1.1 => 2€ * min. qt/basket + 2.1.1 => 1€ * min. qt/basket]
// if basket => 2 x lots total

class ArticleProxyWorth {
  final int lineId;
  final int articleId;
  final int price;
  final int cost;
  final double minimumUnitPerBasket;
  const ArticleProxyWorth(
    this.lineId,
    this.articleId,
    this.price,
    this.cost,
    this.minimumUnitPerBasket,
  );

  static get dummy => ArticleProxyWorth(1, 1, 100, 80, 1);

  ArticleProxyWorth copyWith({
    int? lineId,
    int? articleId,
    int? price,
    int? cost,
    double? minimumUnitPerBasket,
  }) {
    return ArticleProxyWorth(
      lineId ?? this.lineId,
      articleId ?? this.articleId,
      price ?? this.price,
      cost ?? this.cost,
      minimumUnitPerBasket ?? this.minimumUnitPerBasket,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'lineId': lineId,
      'articleId': articleId,
      'price': price,
      'cost': cost,
      'minimumUnitPerBasket': minimumUnitPerBasket,
    };
  }

  factory ArticleProxyWorth.fromMap(Map<String, dynamic> map) {
    return ArticleProxyWorth(
      map['lineId']?.toInt() ?? 0,
      map['articleId']?.toInt() ?? 0,
      map['price']?.toInt() ?? 0,
      map['cost']?.toInt() ?? 0,
      map['minimumUnitPerBasket']?.toDouble() ?? 0.0,
    );
  }

  String toJson() => json.encode(toMap());

  factory ArticleProxyWorth.fromJson(String source) =>
      ArticleProxyWorth.fromMap(json.decode(source));

  @override
  String toString() {
    return 'ArticleProxyWorth(lineId: $lineId, articleId: $articleId, price: $price, cost: $cost, minimumUnitPerBasket: $minimumUnitPerBasket)';
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is ArticleProxyWorth &&
        other.lineId == lineId &&
        other.articleId == articleId &&
        other.price == price &&
        other.cost == cost &&
        other.minimumUnitPerBasket == minimumUnitPerBasket;
  }

  @override
  int get hashCode {
    return lineId.hashCode ^
        articleId.hashCode ^
        price.hashCode ^
        cost.hashCode ^
        minimumUnitPerBasket.hashCode;
  }
}
