import 'dart:convert';

import 'package:collection/collection.dart';

import 'package:models_base/src/base/article_base.dart';
import 'package:models_base/src/base/article_proxy_base.dart';
import 'package:models_base/src/base/article_proxy_worth.dart';

abstract class ItemInCartAbstract<A extends ArticleAbstract,
    W extends ArticleProxyWorth> {
  final A article;
  // below was originally meant to track expiration date
  // turned out nobody ever needed it
  // Now lots are used for a basket of article
  final List<W>? proxiesWorth;
  double quantity;

  ItemInCartAbstract(this.article, this.proxiesWorth, this.quantity);

  @override
  String toString() => "Item($article,$proxiesWorth,$quantity)";

  Map<String, dynamic> toMap() {
    return {
      'article': article.toMap(),
      'proxies': proxiesWorth?.map((x) => x.toMap()).toList(),
      'quantity': quantity,
    };
  }

  String toJson() => json.encode(toMap());

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
    final listEquals = const DeepCollectionEquality().equals;

    return other is ItemInCartAbstract<A, W> &&
        other.article == article &&
        listEquals(other.proxiesWorth, proxiesWorth) &&
        other.quantity == quantity;
  }

  @override
  int get hashCode =>
      article.hashCode ^ proxiesWorth.hashCode ^ quantity.hashCode;
}

class ItemDummy extends ItemInCartAbstract<ArticleDummy, ArticleProxyWorth> {
  ItemDummy()
      : super(
          ArticleDummy(),
          [ArticleProxyWorth.dummy],
          2.0,
        );
}
